<?php
// php reader of AMB files -- turns an AMB book into a web page
//
// Copyright (C) 2020 Mateusz Viste
// http://ambook.sourceforge.net
//
// MIT license
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

$VERSION = "20201207";

function getamafile($ambfname, $amafname) {
  if (! is_file($ambfname)) return(FALSE);
  $fd = fopen($ambfname, "rb");
  if ($fd === FALSE) return(FALSE);
  // read header (AMB1)
  if (fread($fd, 4) !== 'AMB1') return(FALSE);
  // read number of ama files inside
  $fcount = reset(unpack('v', fread($fd, 2)));
  if ($fcount === FALSE) return(FALSE);
  // read index until AMA file is found
  $offset = 0;
  $amalen = 0;
  $amafile = '';
  for ($i = 0; $i < $fcount; $i++) {
    $amafile = rtrim(fread($fd, 12), "\0");
    $offset = reset(unpack('V', fread($fd, 4)));
    $amalen = reset(unpack('v', fread($fd, 2)));
    $bsum = reset(unpack('v', fread($fd, 2)));
    if ($amafile === $amafname) break;
  }
  if ($i >= $fcount) return(FALSE); // not found
  // jump to offset and read file
  fseek($fd, $offset);
  $result = fread($fd, $amalen);
  fclose($fd);
  return($result);
}


if (empty($_GET['fname'])) {
  echo "usage: phpamb.php?fname=file.amb\n";
  exit(0);
}

$ambfname = $_GET['fname'];
$f = 'index.ama'; // default file
if (! empty($_GET['f'])) $f = $_GET['f'];

$title = '';
$title = trim(getamafile($ambfname, 'title'));
if ($title === FALSE) $title = $ambfname;

$ama = getamafile($ambfname, $f);
if ($ama === FALSE) $ama = 'ERROR: FILE NOT FOUND';

echo "<!DOCTYPE html>\n";
echo "<html>\n";
echo "<head>\n";
echo '  <meta charset="UTF-8">' . "\n";
echo "  <title>{$title}</title>\n";
echo '  <link rel="stylesheet" href="phpamb.css">' . "\n";
echo '  <meta name="viewport" content="width=device-width, initial-scale=1">' . "\n";
echo "  <meta name=\"generator\" content=\"phpamb/{$VERSION}\">\n";
echo "</head>\n";
echo "<body>";

echo "<div><span>{$title}</span><span>[{$f}]</span></div>\n";

$amacontent = str_split($ama, 1);
$escnow = 0;  // next char is an escape code
$readlink = 0; // a link target is being read
$opentag = ''; // do I have a currently open html tag?

foreach ($amacontent as $c) {
  // ignore CR
  if ($c == "\r") continue;
  // is link target being read?
  if ($readlink != 0) {
    if (($c == "\n") || ($c == ':')) {
      echo '">';
      $opentag = 'a';
      $readlink = 0;
      if ($c == ':') continue;
    } else {
      echo urlencode($c);
    }
    continue;
  }
  //
  if ($escnow != 0) {
    if ($c == '%') {
      echo '%';
    } else if ($c == 'l') {
      echo '<a href="' . $_SERVER['PHP_SELF'] . "?fname={$ambfname}&amp;f=";
      $readlink = 1;
    } else if ($c == 'h') {
      echo '<h1>';
      $opentag = 'h1';
    }
    $escnow = 0;
    continue;
  }
  // close </a> if open and got LF or new tag
  if ((!empty($opentag)) && (($c == "\n") || ($c == "%"))) {
    echo "</{$opentag}>";
    $opentag = 0;
  }
  //
  if ($c == '%') {
    $escnow = 1;
  } else {
    echo htmlentities($c);
  }
}

echo "</body>\n";
echo "</html>\n";

?>
